@@ -1,6 +1,6 @@
 (******************************************************************************)
 (*     Alt-Ergo: The SMT Solver For Software Verification                     *)
-(*     Copyright (C) 2013-2014 --- OCamlPro                                   *)
+(*     Copyright (C) 2013-2015 --- OCamlPro                                   *)
 (*     This file is distributed under the terms of the CeCILL-C licence       *)
 (******************************************************************************)
 
@@ -22,13 +22,21 @@
 
 type t
 
-type lemma =
-    { qvars: Symbols.Set.t;  (* toplevel quantified variables *)
-      triggers : (Term.t list * Literal.LT.t option) list; (* multi-triggers *)
-      main : t;  (* the main lemma's formula *)
-      name : string; 
-    }
-      
+type binders = (Ty.t * int) Symbols.Map.t (*int tag in globally unique *)
+
+type quantified = {
+  name : string;
+  main : t;
+  triggers : (Term.t list * Literal.LT.t option) list;
+  binders : binders;   (* quantified variable *)
+
+  (* These fields should be (ordered) lists ! important for skolemization *)
+  free_v : Term.t list; (* free variables in main *)
+  free_vty : Ty.t list; (* free type variables in main *)
+  loc : Loc.t; (* location of the "GLOBAL" axiom containing this quantified
+                  formula. It forms with name a unique id *)
+}
+
 and llet = {
   let_var: Symbols.t;
   let_subst : Term.subst;
@@ -36,29 +44,26 @@ and llet = {
   let_f : t;
 }
 
-and skolem = {
-  sko_subst : Term.subst;
-  sko_f : t;
-}
-
-and view = 
+and view =
     Unit of t*t  (* unit clauses *)
   | Clause of t*t      (* a clause (t1 or t2) *)
   | Literal of Literal.LT.t   (* an atom *)
-  | Lemma of lemma   (* a lemma *)
-  | Skolem of skolem  (* lazy substitution *)
+  | Lemma of quantified   (* a lemma *)
+  | Skolem of quantified  (* lazy skolemization *)
   | Let of llet (* a binding of a term *)
 
 
-type gformula = { 
-  f: t; 
-  age: int; 
-  lem: t option; 
+type gformula = {
+  f: t;
+  age: int;
+  lem: t option;
   from_terms : Term.t list;
   mf: bool;
   gf: bool;
 }
 
+val mk_binders : Term.Set.t -> binders
+
 val mk_not : t -> t
 val mk_and : t -> t -> int -> t
 val mk_or : t -> t -> int -> t
@@ -66,11 +71,28 @@ val mk_imp : t -> t -> int -> t
 val mk_if : Term.t -> t -> t -> int -> t
 val mk_iff : t -> t -> int -> t
 val mk_lit : Literal.LT.t -> int -> t
-val mk_forall : Term.Set.t -> Term.Set.t -> 
-  (Term.t list * Literal.LT.t option) list -> t -> string -> int -> t
-val mk_exists : Term.Set.t -> Term.Set.t -> 
-  (Term.t list * Literal.LT.t option) list -> t ->
-  string -> int -> t
+val mk_forall :
+  string -> (* name *)
+  Loc.t -> (* location in the original file *)
+  binders -> (* quantified variables *)
+  (Term.t list * Literal.LT.t option) list -> (* triggers *)
+  t -> (* quantified formula *)
+  int -> (* id, for the GUI *)
+  (Term.t list * Ty.t list) option ->
+  (* free_vars and free_vty: they are computed if None is given *)
+  t
+
+val mk_exists :
+  string -> (* name *)
+  Loc.t -> (* location in the original file *)
+  binders -> (* quantified variables *)
+  (Term.t list * Literal.LT.t option) list -> (* triggers *)
+  t -> (* quantified formula *)
+  int -> (* id, for the GUI *)
+  (Term.t list * Ty.t list) option ->
+  (* free_vars and free_vty: they are computed if None is given *)
+  t
+
 val mk_let : Term.Set.t -> Symbols.t -> Term.t -> t -> int -> t
 
 val add_label : Hstring.t -> t -> unit
@@ -83,10 +105,10 @@ val id : t -> int
 
 val print : Format.formatter -> t -> unit
 
-val terms : t -> Term.Set.t
-val free_vars : t -> Symbols.Set.t
+val ground_terms_rec : t -> Term.Set.t
+val free_vars : t -> Term.Set.t Symbols.Map.t
 
-val apply_subst : Term.subst -> t -> t 
+val apply_subst : Term.subst -> t -> t
 
 val compare : t -> t -> int
 val equal : t -> t -> bool
@@ -94,6 +116,7 @@ val hash : t -> int
 val vrai : t
 val faux : t
 
+val skolemize : quantified -> t
+
 module Set : Set.S with type elt = t
 module Map : Map.S with type key = t
-
