@@ -1,6 +1,6 @@
 (******************************************************************************)
 (*     Alt-Ergo: The SMT Solver For Software Verification                     *)
-(*     Copyright (C) 2013-2014 --- OCamlPro                                   *)
+(*     Copyright (C) 2013-2015 --- OCamlPro                                   *)
 (*     This file is distributed under the terms of the CeCILL-C licence       *)
 (******************************************************************************)
 
@@ -28,9 +28,9 @@ module Q = Numbers.Q
 
 module Ex = Explanation
 
-type borne = 
-  | Strict of (Q.t * Ex.t) 
-  | Large of (Q.t * Ex.t) 
+type borne =
+  | Strict of (Q.t * Ex.t)
+  | Large of (Q.t * Ex.t)
   | Pinfty | Minfty
 
 let compare_bornes b1 b2 =
@@ -38,14 +38,14 @@ let compare_bornes b1 b2 =
     | Minfty, Minfty | Pinfty, Pinfty -> 0
     | Minfty, _ | _, Pinfty -> -1
     | Pinfty, _ | _, Minfty -> 1
-    | Strict (v1, _), Strict (v2, _) | Large (v1, _), Large (v2, _) 
-    | Strict (v1, _), Large (v2, _) | Large (v1, _), Strict (v2, _) -> 
+    | Strict (v1, _), Strict (v2, _) | Large (v1, _), Large (v2, _)
+    | Strict (v1, _), Large (v2, _) | Large (v1, _), Strict (v2, _) ->
       Q.compare v1 v2
 
 let compare_bu_bl b1 b2 =
   match b1, b2 with
     | (Minfty | Pinfty), _ | _,(Minfty | Pinfty)
-    | Large _, Large _ -> 
+    | Large _, Large _ ->
       compare_bornes b1 b2
     | Strict _, Strict _ ->
       let c = compare_bornes b1 b2 in
@@ -53,11 +53,11 @@ let compare_bu_bl b1 b2 =
     | Strict (v1, _), Large (v2, _) | Large (v1, _), Strict (v2, _) ->
       let c = Q.compare v1 v2 in
       if c = 0 then -1 else c
-        
+
 let compare_bl_bu b1 b2 =
   match b1, b2 with
     | (Minfty | Pinfty), _ | _,(Minfty | Pinfty)
-    | Large _, Large _ -> 
+    | Large _, Large _ ->
       compare_bornes b1 b2
     | Strict _, Strict _ ->
       let c = compare_bornes b1 b2 in
@@ -66,11 +66,11 @@ let compare_bl_bu b1 b2 =
       let c = Q.compare v1 v2 in
       if c = 0 then 1 else c
 
-let compare_bl_bl b1 b2 = 
-  match b1, b2 with 
-    | (Minfty | Pinfty), _ | _,(Minfty | Pinfty) 
-    | Strict _, Strict _ | Large _, Large _ -> 
-      compare_bornes b1 b2 
+let compare_bl_bl b1 b2 =
+  match b1, b2 with
+    | (Minfty | Pinfty), _ | _,(Minfty | Pinfty)
+    | Strict _, Strict _ | Large _, Large _ ->
+      compare_bornes b1 b2
     | Strict (v1, _), Large (v2, _) ->
       let c = Q.compare v1 v2 in
       if c = 0 then 1 else c
@@ -80,9 +80,9 @@ let compare_bl_bl b1 b2 =
 
 let compare_bu_bu b1 b2 =
   match b1, b2 with
-    | (Minfty | Pinfty), _ | _,(Minfty | Pinfty) 
-    | Strict _, Strict _ | Large _, Large _ -> 
-      compare_bornes b1 b2 
+    | (Minfty | Pinfty), _ | _,(Minfty | Pinfty)
+    | Strict _, Strict _ | Large _, Large _ ->
+      compare_bornes b1 b2
     | Strict (v1, _), Large (v2, _) ->
       let c = Q.compare v1 v2 in
       if c = 0 then -1 else c
@@ -90,7 +90,7 @@ let compare_bu_bu b1 b2 =
       let c = Q.compare v1 v2 in
       if c = 0 then 1 else c
 
-type t = { 
+type t = {
   ints : (borne * borne) list;
   is_int : bool;
   expl: Ex.t
@@ -105,32 +105,32 @@ module Debug = struct
     | Minfty -> fprintf fmt "%s" (if pretty then "-∞" else "-inf")
     | Pinfty -> fprintf fmt "%s" (if pretty then "+∞" else "+inf")
     | Strict (v, e) | Large (v, e) ->
-      if verbose () || proof () then 
-        fprintf fmt "%s %a" (Q.string_of v) Ex.print e
-      else fprintf fmt "%s" (Q.string_of v)
-        
+      if verbose () || proof () then
+        fprintf fmt "%s %a" (Q.to_string v) Ex.print e
+      else fprintf fmt "%s" (Q.to_string v)
+
   let print_interval pretty fmt (b1,b2) =
     let c1, c2 = match b1, b2 with
       | Large _, Large _ -> '[', ']'
       | Large _, _ -> '[', '['
       | _, Large _ -> ']', ']'
       | _, _ -> ']', '['
-    in 	    
-    fprintf fmt "%c%a;%a%c" 
+    in
+    fprintf fmt "%c%a;%a%c"
       c1 (print_borne pretty) b1 (print_borne pretty) b2 c2
 
   let rec print_list pretty fmt = function
     | [] -> ()
     | [t] -> print_interval pretty fmt t
-    | t::l -> fprintf fmt "%a%s%a" 
-      (print_interval pretty) t 
+    | t::l -> fprintf fmt "%a%s%a"
+      (print_interval pretty) t
       (if pretty then " ∪ " else " ") (print_list pretty) l
 
-  let pretty_print fmt {ints = ints; is_int = b; expl = e } = 
+  let pretty_print fmt {ints = ints; is_int = b; expl = e } =
     print_list true fmt ints;
     if verbose () || proof () then fprintf fmt " %a" Ex.print e
-      
-  let print fmt {ints = ints; is_int = b; expl = e } = 
+
+  let print fmt {ints = ints; is_int = b; expl = e } =
     print_list false fmt ints;
     if verbose () || proof () then fprintf fmt " %a" Ex.print e
 
@@ -147,7 +147,7 @@ let undefined ty = {
 }
 
 let point b ty e = {
-  ints = [Large (b, e), Large (b, e)]; 
+  ints = [Large (b, e), Large (b, e)];
   is_int = ty  = Ty.Tint;
   expl = Ex.empty
 }
@@ -171,7 +171,7 @@ let is_point { ints = l; expl = e } =
     | _ -> None
 
 let add_expl_zero i expl =
-  let res = List.map (fun x -> 
+  let res = List.map (fun x ->
     match x with
       | (Large (c1, e1) , Large (c2, e2)) when Q.sign c1 = 0 && Q.sign c2 = 0->
         (Large (Q.zero, Ex.union e1 expl),
@@ -182,20 +182,20 @@ let add_expl_zero i expl =
 let check_one_interval b1 b2 is_int =
   match b1, b2 with
     | Pinfty, _ | _, Minfty  -> raise (EmptyInterval Ex.empty)
-    | (Strict (v1, e1) | Large (v1,e1)), 
+    | (Strict (v1, e1) | Large (v1,e1)),
       (Strict (v2, e2) | Large (v2, e2)) ->
-      let c = Q.compare v1 v2 in 
-      if c > 0 then raise 
+      let c = Q.compare v1 v2 in
+      if c > 0 then raise
 	(EmptyInterval (Ex.union e2 e1));
       if c = 0 then begin
 	match b1, b2 with
-	  | Large _, Large _ when not is_int || Q.is_integer v1 ->
+	  | Large _, Large _ when not is_int || Q.is_int v1 ->
 	    ()
 	  | _ -> raise (EmptyInterval (Ex.union e2 e1))
       end
     | _ -> ()
 
-let min_borne b1 b2 = 
+let min_borne b1 b2 =
   match b1, b2 with
     | Minfty , _ | _ , Minfty -> Minfty
     | b , Pinfty | Pinfty, b -> b
@@ -203,31 +203,31 @@ let min_borne b1 b2 =
       let c = Q.compare v1 v2 in
       if c < 0 then b1
       else if c > 0 then b2
-      else match b1, b2 with 
+      else match b1, b2 with
 	| (Strict _ as b) , _ | _, (Strict _ as b) -> b
 	| _, _ -> b1
-          
-let max_borne b1 b2 = 
+
+let max_borne b1 b2 =
   match b1, b2 with
     | Pinfty , _ | _ , Pinfty -> Pinfty
     | b , Minfty | Minfty, b -> b
-    | (Strict (v1, _) | Large (v1, _)) , (Strict (v2, _) | Large (v2, _)) -> 
+    | (Strict (v1, _) | Large (v1, _)) , (Strict (v2, _) | Large (v2, _)) ->
       let c = Q.compare v1 v2 in
       if c > 0 then b1
       else if c < 0 then b2
-      else match b1, b2 with 
+      else match b1, b2 with
 	| (Strict _ as b) , _ | _, (Strict _ as b) -> b
 	| _, _ -> b1
-	  
+
 let pos_borne b1 =
   compare_bornes b1 (borne_of true Ex.empty Q.zero) >= 0
-let pos_borne_strict b1 = 
+let pos_borne_strict b1 =
   compare_bornes b1 (borne_of true Ex.empty Q.zero) > 0
-let neg_borne b1 = 
+let neg_borne b1 =
   compare_bornes b1 (borne_of true Ex.empty Q.zero) <= 0
-let neg_borne_strict b1 = 
+let neg_borne_strict b1 =
   compare_bornes b1 (borne_of true Ex.empty Q.zero) < 0
-let zero_borne b1 = 
+let zero_borne b1 =
   compare_bornes b1 (borne_of true Ex.empty Q.zero) = 0
 
 exception Found of Sig.answer
@@ -235,12 +235,12 @@ exception Found of Sig.answer
 let doesnt_contain_0 {ints=l} =
   try
     let max = List.fold_left
-      (fun old_u (l, u) -> 
+      (fun old_u (l, u) ->
 	if neg_borne l && pos_borne u then raise (Found Sig.No);
-	if neg_borne_strict old_u && pos_borne_strict l then 
-	  raise (Found 
-		   (Sig.Yes 
-		      (Ex.union 
+	if neg_borne_strict old_u && pos_borne_strict l then
+	  raise (Found
+		   (Sig.Yes
+		      (Ex.union
 			 (explain_borne old_u) (explain_borne l), [])));
 	u) Minfty l in
     if neg_borne_strict max then Sig.Yes (explain_borne max, [])
@@ -258,11 +258,11 @@ let is_strict_smaller i1 i2 =
 	  then raise Exit
 	) i1 i2;
 	false
-      with 
+      with
 	| Exit -> true
 	| Invalid_argument _ -> true
 
-let is_strict_smaller {ints=i1} {ints=i2} = 
+let is_strict_smaller {ints=i1} {ints=i2} =
   is_strict_smaller i1 i2
 
 
@@ -286,7 +286,7 @@ let add_borne b1 b2 =
     | Minfty, Pinfty | Pinfty, Minfty -> assert false
     | Minfty, _ | _, Minfty -> Minfty
     | Pinfty, _ | _, Pinfty -> Pinfty
-    | Large (v1, e1), Large (v2, e2) -> 
+    | Large (v1, e1), Large (v2, e2) ->
       Large (Q.add v1 v2, Ex.union e1 e2)
     | (Large (v1, e1) | Strict (v1, e1)), (Large (v2, e2) | Strict (v2, e2)) ->
       Strict (Q.add v1 v2, Ex.union e1 e2)
@@ -299,9 +299,9 @@ let add_interval l (b1,b2) =
     ) l []
 
 let add {ints = l1; is_int = is_int; expl = e1} {ints = l2; expl = e2}=
-  let l = 
+  let l =
     List.fold_left
-      (fun l bs -> let i = add_interval l1 bs in i@l) [] l2 
+      (fun l bs -> let i = add_interval l1 bs in i@l) [] l2
   in
   union { ints = l ; is_int = is_int; expl = Ex.union e1 e2 }
 
@@ -313,7 +313,7 @@ let minus_borne = function
 
 let scale_borne n b =
   assert (Q.sign n >= 0);
-  if Q.sign n = 0 then 
+  if Q.sign n = 0 then
     match b with
       | Pinfty | Minfty -> Large (Q.zero, Ex.empty)
       | Large (_, e) | Strict (_, e) ->  Large (Q.zero, e)
@@ -333,24 +333,28 @@ let scale n uints =
   Options.tool_req 4 "TR-Arith-Axiomes scale";
   let l = List.map (scale_interval n) uints.ints in
   union { uints with ints = l; expl = uints.expl }
-    
+
 let mult_borne b1 b2 =
   match b1,b2 with
     | Minfty, Pinfty | Pinfty, Minfty -> assert false
     | Minfty, b | b, Minfty ->
-      if compare_bornes b (borne_of true Ex.empty Q.zero) = 0 
+      if compare_bornes b (borne_of true Ex.empty Q.zero) = 0
       then b
       else if pos_borne b then Minfty
       else Pinfty
     | Pinfty, b | b, Pinfty ->
-      if compare_bornes b (borne_of true Ex.empty Q.zero) = 0 
+      if compare_bornes b (borne_of true Ex.empty Q.zero) = 0
       then b
       else if pos_borne b then Pinfty
       else Minfty
+    | Strict (_, e1), Large (v, e2)
+    | Large (v, e1), Strict (_, e2) when Q.is_zero v ->
+      Large (Q.zero, Ex.union e1 e2)
+
     | Strict (v1, e1), Strict (v2, e2) | Strict (v1, e1), Large (v2, e2)
-    | Large (v1, e1), Strict (v2, e2) -> 
+    | Large (v1, e1), Strict (v2, e2) ->
       Strict (Q.mult v1 v2, Ex.union e1 e2)
-    | Large (v1, e1), Large (v2, e2) -> 
+    | Large (v1, e1), Large (v2, e2) ->
       Large (Q.mult v1 v2, Ex.union e1 e2)
 
 let mult_borne_inf b1 b2 =
@@ -363,10 +367,10 @@ let mult_borne_sup b1 b2 =
     | Minfty, Pinfty | Pinfty, Minfty -> Pinfty
     | _, _ -> mult_borne b1 b2
 
-type interval_class = 
-  | P of Ex.t 
-  | M of Ex.t 
-  | N of Ex.t 
+type interval_class =
+  | P of Ex.t
+  | M of Ex.t
+  | N of Ex.t
   | Z
 
 let class_of (l,u) =
@@ -376,19 +380,19 @@ let class_of (l,u) =
   else M (Ex.union (explain_borne l) (explain_borne u))
 
 let mult_bornes (a,b) (c,d) =
-  (* see util/intervals_mult.png *)
+  (* see ../extra/intervals_mult.png *)
   match class_of (a,b), class_of (c,d) with
-    | P e1, P e2 -> 
+    | P e1, P e2 ->
       mult_borne_inf a c, mult_borne_sup b d, Ex.union e1 e2
-    | P e1, M e2 -> 
+    | P e1, M e2 ->
       mult_borne_inf b c, mult_borne_sup b d, Ex.union e1 e2
-    | P e1, N e2 -> 
+    | P e1, N e2 ->
       mult_borne_inf b c, mult_borne_sup a d, Ex.union e1 e2
-    | M e1, P e2 -> 
+    | M e1, P e2 ->
       mult_borne_inf a d, mult_borne_sup b d, Ex.union e1 e2
-    | M e1, M e2 -> 
+    | M e1, M e2 ->
       min_borne (mult_borne_inf a d) (mult_borne_inf b c),
-      max_borne (mult_borne_sup a c) (mult_borne_sup b d), 
+      max_borne (mult_borne_sup a c) (mult_borne_sup b d),
       Ex.union e1 e2
     | M e1, N e2 ->
       mult_borne_inf b c, mult_borne_sup a c, Ex.union e1 e2
@@ -400,7 +404,7 @@ let mult_bornes (a,b) (c,d) =
       mult_borne_inf b d, mult_borne_sup a c, Ex.union e1 e2
     | Z, (P _ | M _ | N _ | Z) -> (a, b, Ex.empty)
     | (P _ | M _ | N _ ), Z -> (c, d, Ex.empty)
-      
+
 let rec power_borne_inf p b =
   match p with
     | 1 -> b
@@ -436,14 +440,14 @@ let power_bornes p (b1,b2) =
       | p when p mod 2 = 0 -> (power_borne_inf p b2, power_borne_sup p b1)
       | _ -> (power_borne_inf p b1, power_borne_sup p b2)
   else assert false
-    
+
 let int_of_borne_inf b =
   match b with
     | Minfty | Pinfty -> b
     | Large (v, e) -> Large (Q.ceiling v, e)
     | Strict (v, e) ->
       let v' = Q.ceiling v in
-      if Q.compare v' v > 0 then Large (v', e) else Large (Q.add v Q.one, e) 
+      if Q.compare v' v > 0 then Large (v', e) else Large (Q.add v Q.one, e)
 
 let int_of_borne_sup b =
   match b with
@@ -451,7 +455,7 @@ let int_of_borne_sup b =
     | Large (v, e) -> Large (Q.floor v, e)
     | Strict (v, e) ->
       let v' = Q.floor v in
-      if Q.compare v' v < 0 then Large (v', e) else Large (Q.sub v Q.one, e) 
+      if Q.compare v' v < 0 then Large (v', e) else Large (Q.sub v Q.one, e)
 
 let int_div_of_borne_inf b =
   match b with
@@ -459,7 +463,7 @@ let int_div_of_borne_inf b =
     | Large (v, e) -> Large (Q.floor v, e)
     | Strict (v, e) ->
       let v' = Q.floor v in
-      if Q.compare v' v > 0 then Large (v', e) else Large (Q.add v Q.one, e) 
+      if Q.compare v' v > 0 then Large (v', e) else Large (Q.add v Q.one, e)
 
 let int_div_of_borne_sup b =
   match b with
@@ -469,10 +473,10 @@ let int_div_of_borne_sup b =
       let v' = Q.floor v in
       if Q.compare v' v < 0 then Large (v', e) else Large (Q.sub v Q.one, e)
 
-let int_bornes l u = 
+let int_bornes l u =
   int_of_borne_inf l, int_of_borne_sup u
 
-let int_div_bornes l u = 
+let int_div_bornes l u =
   int_div_of_borne_inf l, int_div_of_borne_sup u
 
 
@@ -481,7 +485,7 @@ let intersect ({ints=l1; expl=e1; is_int=is_int} as uints1)
   let rec step (l1,l2) acc expl =
     match l1, l2 with
       | (lo1,up1)::r1, (lo2,up2)::r2 ->
-	let (lo1,up1), (lo2,up2) = 
+	let (lo1,up1), (lo2,up2) =
 	  if is_int then (int_bornes lo1 up1), (int_bornes lo2 up2)
 	  else (lo1,up1), (lo2,up2) in
 	let cll = compare_bl_bl lo1 lo2 in
@@ -496,7 +500,7 @@ let intersect ({ints=l1; expl=e1; is_int=is_int} as uints1)
 	      let lor1 = add_expl_to_borne lor1 nexpl in
 	      let r1 = (lor1,upr1)::rr1 in
 	      step (r1, l2) acc expl
-	else if clu > 0 then 
+	else if clu > 0 then
 	  let nexpl  = Ex.union (explain_borne up2) (explain_borne lo1) in
 	  match r2 with
 	    | [] -> step (l1, r2) acc (Ex.union nexpl expl)
@@ -507,13 +511,13 @@ let intersect ({ints=l1; expl=e1; is_int=is_int} as uints1)
 	(* incorrect *)
 	(* else if cll = 0 && cuu = 0 then  *)
 	(*   step (r1, r2) ((lo1,up1)::acc) expl *)
-	else if cll <= 0 && cuu >= 0 then 
+	else if cll <= 0 && cuu >= 0 then
 	  step (l1, r2) ((lo2,up2)::acc) expl
-	else if cll >= 0 && cuu <= 0 then 
+	else if cll >= 0 && cuu <= 0 then
 	  step (r1, l2) ((lo1,up1)::acc) expl
-	else if cll <= 0 && cuu <= 0 && cul >= 0 then 
+	else if cll <= 0 && cuu <= 0 && cul >= 0 then
 	  step (r1, l2) ((lo2,up1)::acc) expl
-	else if cll >= 0 && cuu >= 0 && clu <= 0 then 
+	else if cll >= 0 && cuu >= 0 && clu <= 0 then
 	  step (l1, r2) ((lo1,up2)::acc) expl
 	else assert false
             | [], _ | _, [] ->  List.rev acc, expl
@@ -524,13 +528,13 @@ let intersect ({ints=l1; expl=e1; is_int=is_int} as uints1)
 
 
 let new_borne_sup expl b ~is_le uints =
-  intersect 
+  intersect
     { ints = [Minfty, (borne_of is_le expl b)];
       is_int = uints.is_int;
       expl = Ex.empty } uints
 
 let new_borne_inf expl b ~is_le uints =
-  intersect 
+  intersect
     { ints = [(borne_of is_le expl b), Pinfty];
       is_int = uints.is_int;
       expl = Ex.empty } uints
@@ -549,29 +553,29 @@ let complement ({ints=l; expl=e} as uints) =
 	  | _ -> b2 in
 	if bu = Minfty then step r bl acc
 	else step r bl ((prev, bu)::acc)
-      | [] -> 
+      | [] ->
 	if prev = Pinfty then List.rev acc
 	else List.rev ((prev, Pinfty)::acc)
   in
   { uints with ints = step l Minfty [] }
-    
+
 
 let exclude uints1 uints2 =
-  intersect (complement uints1) uints2 
+  intersect (complement uints1) uints2
 
 let mult u1 u2 =
   Options.tool_req 4 "TR-Arith-Axiomes mult";
-  let resl, expl = 
+  let resl, expl =
     List.fold_left
       (fun (l', expl) b1 ->
-	List.fold_left 
+	List.fold_left
 	  (fun (l, ex) b2 ->
 	    let bl, bu, ex' = mult_bornes b1 b2 in
 	    (bl, bu)::l, Ex.union ex ex') (l', expl) u2.ints)
       ([], Ex.empty) u1.ints
   in
   union { ints=resl; is_int = u1.is_int;
-	  expl = Ex.union expl 
+	  expl = Ex.union expl
       (Ex.union u1.expl u2.expl) }
 
 let power n u =
@@ -579,17 +583,17 @@ let power n u =
   let l = List.map (power_bornes n) u.ints in
   union { u with ints = l }
 
-let root_num a n = 
+let root_num a n =
   if Q.sign a < 0 then assert false
   else if Q.sign a = 0 then Q.zero
   else
-    let v = Q.float_of a in
+    let v = Q.to_float a in
     let w = if v < min_float then min_float
       else if v > max_float then max_float
       else v
     in
-    if n = 2 then Q.of_float (sqrt w)
-    else Q.of_float (w ** (1./. (float n)))
+    if n = 2 then Q.from_float (sqrt w)
+    else Q.from_float (w ** (1./. (float n)))
 
 let root_default_num a n =
   let s = root_num a n in
@@ -663,7 +667,7 @@ let rec root n ({ints = l; is_int = is_int; expl = e} as u) =
 	  (root_interval is_int bs n)@l'
 	) [] l in
     union { ints = l; is_int = is_int; expl = e }
-      
+
 let finite_size {ints = l; is_int = is_int} =
   if (not is_int) then None
   else
@@ -673,24 +677,27 @@ let finite_size {ints = l; is_int = is_int} =
 	  (fun n (b1,b2) ->
 	    match b1, b2 with
 	      | Minfty, _ | _, Pinfty -> raise Exit
-	      | Large (v1, _) , Large (v2, _) -> 
+	      | Large (v1, _) , Large (v2, _) ->
                 Q.add n (Q.add (Q.sub v2 v1) Q.one)
 	      | _, _ -> assert false
 	  ) Q.zero l in
       Some n
     with Exit -> None
-      
+
 let borne_inf = function
   | {ints = (Large (v, ex), _)::_} -> v, ex
   | _ -> invalid_arg "Intervals.borne_inf : No finite lower bound"
 
-
+let borne_sup {ints=ints} =
+  match List.rev ints with
+  | (_, Large (v, ex))::_ -> v, ex
+  | _ -> invalid_arg "Intervals.borne_sup : No finite upper bound"
 
 let inv_borne_inf b is_int ~other =
   match b with
     | Pinfty -> assert false
     | Minfty ->
-      if is_int then Large (Q.zero,  explain_borne other) 
+      if is_int then Large (Q.zero,  explain_borne other)
       else Strict (Q.zero, explain_borne other)
     | Strict (c, e) | Large (c, e) when Q.sign c = 0 -> Pinfty
     | Strict (v, e) -> Strict (Q.div Q.one v, e)
@@ -712,10 +719,10 @@ let inv_bornes (l, u) is_int =
 
 let inv ({ints=l; is_int=is_int} as u) =
   try
-    let l' = List.fold_left 
+    let l' = List.fold_left
       (fun acc (l,u) ->
-	if (pos_borne_strict l && pos_borne_strict u) 
-	  || (neg_borne_strict l && neg_borne_strict u) then 
+	if (pos_borne_strict l && pos_borne_strict u)
+	  || (neg_borne_strict l && neg_borne_strict u) then
 	  (inv_bornes (l, u) is_int) :: acc
 	else raise Exit
       ) [] l in
@@ -732,8 +739,8 @@ let div i1 i2 =
       | Sig.No -> i1
     in
     let ({ints=l; is_int=is_int} as i) = mult i1 inv_i2 in
-    let l = 
-      if is_int then 
+    let l =
+      if is_int then
 	List.map (fun (l,u) -> int_div_bornes l u) l
       else l in
     { i with ints = l }
