@@ -1,6 +1,6 @@
 (******************************************************************************)
 (*     Alt-Ergo: The SMT Solver For Software Verification                     *)
-(*     Copyright (C) 2013-2014 --- OCamlPro                                   *)
+(*     Copyright (C) 2013-2015 --- OCamlPro                                   *)
 (*     This file is distributed under the terms of the CeCILL-C licence       *)
 (******************************************************************************)
 
@@ -44,7 +44,8 @@ val set_debug_combine : bool -> unit
 val set_debug_proof : bool -> unit
 val set_debug_split : bool -> unit
 val set_debug_matching : bool -> unit
-val set_profiling : bool -> unit
+val set_timers : bool -> unit
+val set_profiling : float -> bool -> unit
 
 (** additional setters *)
 
@@ -58,7 +59,7 @@ val set_triggers_var : bool -> unit
 val set_nb_triggers : int -> unit
 val set_greedy : bool -> unit
 val set_select : int -> unit
-val set_no_rm_eq_existential : bool -> unit
+val set_rm_eq_existential : bool -> unit
 val set_no_Ematching : bool -> unit
 val set_nocontracongru : bool -> unit
 val set_term_like_pp : bool -> unit
@@ -73,16 +74,26 @@ val set_restricted : bool -> unit
 val set_bottom_classes : bool -> unit
 val set_timelimit : float -> unit
 val set_thread_yield : (unit -> unit) -> unit
-val set_timer_start : (Timers.kind -> unit) -> unit
-val set_timer_pause : (Timers.kind -> unit) -> unit
+
+(** This functions assumes (asserts) that timers() yields true **)
+val set_timer_start : (Timers.ty_module -> Timers.ty_function -> unit) -> unit
+
+(** This functions assumes (asserts) that timers() yields true **)
+val set_timer_pause : (Timers.ty_module -> Timers.ty_function -> unit) -> unit
 val set_timeout : (unit -> unit) -> unit
 val set_partial_bmodel : bool -> unit
+val set_save_used_context : bool -> unit
+
+
+(* updates the filename to be parsed and sets a js_mode flag *)
+val set_file_for_js : string -> unit
 
 
 (** getter functions **********************************************************)
 
 (** getters for debug flags *)
 val debug : unit -> bool
+val debug_warnings : unit -> bool
 val debug_cc : unit -> bool
 val debug_use : unit -> bool
 val debug_uf : unit -> bool
@@ -107,6 +118,9 @@ val enable_assertions : unit -> bool
 val type_only : unit -> bool
 val parse_only : unit -> bool
 val steps_bound : unit -> int
+val no_tcp : unit -> bool
+val no_theory : unit -> bool
+val tighten_vars : unit -> bool
 val age_bound : unit -> int
 val notriggers : unit -> bool
 val triggers_var : unit -> bool
@@ -114,7 +128,7 @@ val nb_triggers : unit -> int
 val verbose : unit -> bool
 val greedy : unit -> bool
 val select : unit -> int
-val no_rm_eq_existential : unit -> bool
+val rm_eq_existential : unit -> bool
 val no_Ematching : unit -> bool
 val nocontracongru : unit -> bool
 val term_like_pp : unit -> bool
@@ -129,9 +143,11 @@ val restricted : unit -> bool
 val bottom_classes : unit -> bool
 val timelimit : unit -> float
 val profiling : unit -> bool
-val satmode : unit -> bool
-val smt2file : unit -> bool
-val smtfile : unit -> bool
+val profiling_period : unit -> float
+val js_mode : unit -> bool
+
+(** this option also yields true if profiling is set to true **)
+val timers : unit -> bool
 val replay : unit -> bool
 val replay_used_context : unit -> bool
 val replay_all_used_context : unit -> bool
@@ -141,13 +157,32 @@ val get_file : unit -> string
 val get_session_file : unit -> string
 val get_used_context_file : unit -> string
 val sat_plugin : unit -> string
+val inequalities_plugin : unit -> string
+val profiling_plugin : unit -> string
 val normalize_instances : unit -> bool
 val partial_bmodel : unit -> bool
+val backward_compat : unit -> bool
 
 (** particular getters : functions that are immediately executed **************)
 val exec_thread_yield : unit -> unit
-val exec_timer_start : Timers.kind -> unit
-val exec_timer_pause : Timers.kind -> unit
+val exec_timer_start : Timers.ty_module -> Timers.ty_function -> unit
+val exec_timer_pause : Timers.ty_module -> Timers.ty_function -> unit
 val exec_timeout : unit -> unit
 
 val tool_req : int -> string -> unit
+
+(** Simple Timer module **)
+module Time : sig
+
+  val start : unit -> unit
+  val value : unit -> float
+
+  val set_timeout : unit -> unit
+  val unset_timeout : unit -> unit
+
+end
+
+(** globals **)
+
+val cs_steps : unit -> int
+val incr_cs_steps : unit -> unit
